#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"column.hpp"
#include	"transfrm.hpp"

/* ---------------------------------------------------------------------
	Initialise the transformation to do an empty (null) translation.
   --------------------------------------------------------------------- */

void	Transform::Initialise(int iRecLen, int iNewRecLen)
{
	int	i;

	this->iNewRecLen = iNewRecLen;

	/* allocate memory to keep track of original and new column sequence */

	if ((piOriginal = new int[iRecLen]) == 0)
		error_handler(4, "first permutation table");

	for (i=0; i<iRecLen; i++) 
		piOriginal[i] = i+1;

	if ((piPermute = new int[iNewRecLen]) == 0)
		error_handler(4, "second permutation table");

	for (i=0; i<iNewRecLen; i++) 
		piPermute[i] = 0;

	/*	allocate memory to allow modification of selected columns */

	if ((pchSet = (char*) new char[iNewRecLen]) == 0)
		error_handler(4, "columns to set");

	for (i=0; i<iNewRecLen; i++) 
		pchSet[i] = 0;

	if ((pchValues = (char*) new char[iNewRecLen]) == 0)
		error_handler(4, "new column values");

	for (i=0; i<iNewRecLen; i++) 
		pchValues[i] = 0;
}

/* ---------------------------------------------------------------------
	Read the command line arguments to determine how to transform each
	column within the record.
   --------------------------------------------------------------------- */

void	Transform::Setup(int iRecLen, int argc, char** argv)
{
	int	i, j;
	int	*piDest, *piSource;
	int	cStart, cEnd, cNew;
	char	*parm;
	char	chType;

	for (i=3; i<argc; i++) {
		if (toupper(*argv[i]) == 'R') {
			parm		= argv[i]+1;
			cStart	= read_param(&parm, 1);
			cEnd		= read_param(&parm, iRecLen);
			chType	= *parm++;

			switch (toupper(chType)) {
				case 'C'	:
					if (cStart < 1 || cStart > iRecLen)
						error_handler(9,argv[i]);

					if (cEnd < 1 || cEnd > iRecLen)
						error_handler(9,argv[i]);

					cNew = read_param(&parm, 1);
					if (cNew < 1 || cNew > iNewRecLen)
						error_handler(9,argv[i]);

					piDest = piPermute + cNew-1;
					for (j=cStart; j<=cEnd; j++) {
						if (*piDest != 0) error_handler(8,argv[i]);
						*piDest++ = j;
					}
					break;

				case 'M' :
					if (cStart < 1 || cStart > iRecLen)
						error_handler(9,argv[i]);

					if (cEnd < 1 || cEnd > iRecLen)
						error_handler(9,argv[i]);

					cNew = read_param(&parm, 1);
					if (cNew < 1 || cNew > iNewRecLen)
						error_handler(9,argv[i]);

					piDest	= piPermute		+ cNew	- 1;
					piSource	= piOriginal	+ cStart	- 1;

					for (j=cStart; j<=cEnd; j++) {
						if (*piDest != 0) error_handler(8,argv[i]);
						*piDest++	= j;
						*piSource++	= 0;
					}
					break;

				case 'D' :
					if (cStart < 1 || cStart > iRecLen)
						error_handler(9,argv[i]);

					if (cEnd < 1 || cEnd > iRecLen)
						error_handler(9,argv[i]);

					piSource = piOriginal + cStart-1;
					for (j=cStart; j<=cEnd; j++) {
						if (*piSource == 0) error_handler(8,argv[i]);
						*piSource++ = 0;
					}
					break;

				case 'Z' :
					if (cStart < 1 || cStart > iNewRecLen)
						error_handler(12,argv[i]);

					if (cEnd < 1 || cEnd > iNewRecLen)
						error_handler(12,argv[i]);

					for (j=cStart-1; j<cEnd; j++) {
						pchSet[j]		= 1;
						pchValues[j]	= 0;
					}
					break;

				case 'B' :
					if (cStart < 1 || cStart > iNewRecLen)
						error_handler(12,argv[i]);

					if (cEnd < 1 || cEnd > iNewRecLen)
						error_handler(12,argv[i]);

					for (j=cStart-1; j<cEnd; j++) {
						pchSet[j]		= 1;
						pchValues[j]	= ' ';
					}
					break;

				case 'S' :
					if (cStart < 1 || cStart > iNewRecLen)
						error_handler(12,argv[i]);

					if (cEnd < 1 || cEnd > iNewRecLen)
						error_handler(12,argv[i]);

					for (j=cStart-1; j<cEnd; j++) {
						pchSet[j]		= 1;
						pchValues[j]	= (char) read_param(&parm, 0);
					}
					break;

				default	:
					error_handler(5,argv[i]);
					break;
			}
		}
	}

	piSource	= piOriginal;
	piDest	= piPermute - 1;

	for (i=0; i<iRecLen; i++) {
		if (*piSource != 0) {
			while (*++piDest);
			*piDest = *piSource;
		}
		piSource++;
	}
}

/* ---------------------------------------------------------------------
	Re-arrange the columns of a record.
   --------------------------------------------------------------------- */

void	Transform::translate(const char *pchInput, char *pchOutput, BOOL fTextMode)
{
	int	i, j;
	char	ch;

	for (i=0; i<iNewRecLen; i++) {
		j = piPermute[i];
		ch = (j == 0) ? 0 : pchInput[j-1];
		if ((ch == 0) && fTextMode) break;
		*pchOutput++ = ch;
	}
	if (fTextMode) *pchOutput = 0;
}

/* ---------------------------------------------------------------------
	Replace selected columns of a record.
   --------------------------------------------------------------------- */

void	Transform::replace(char *pchOutput)
{
	int		i;

	for (i=0; i<iNewRecLen; i++) {
		if (pchSet[i])
			pchOutput[i] = pchValues[i];
	}
}

/* ---------------------------------------------------------------------
	Perform the complete transformation of a record.
   --------------------------------------------------------------------- */

void	Transform::Record(const char *pchInput, char *pchOutput, BOOL fTextMode)
{
	memset(pchOutput, 0, iNewRecLen);
	translate(pchInput, pchOutput, fTextMode);
	replace(pchOutput);
}

/* ---------------------------------------------------------------------
	Free any dynamically allocated structures.
   --------------------------------------------------------------------- */

void	Transform::Cleanup()
{
	delete[] piOriginal;
	delete[] piPermute;
	delete[] pchSet;
	delete[] pchValues;
}
